; **********************************************************************
; SCRIPT NAME      : Trapper.Sc
; WRITTEN BY       : Joshua Evans
; LAST MODIFCATION : 9/8/91
; PURPOSE          : An Interactive Demonstration of the new Event-driven                                                  
;                  : programming paradigm in Paradox 4.0
; PARADOX VERSION  : 4.0
; RETURN VALUE     : 
; NOTES            : 1. Version 1.1, later version than the one from the 
;                  :    Palm Desert Database conference. 
;                  : 2. Thanks to Dan Bernstein for some helpful code and 
;                  :    necessary input. 
; **********************************************************************

PROC About()
;Procedure to call the About box which describes this application
PRIVATE OKValue
  
  SHOWPULLDOWN ENDMENU
  PROMPT ""
  SHOWDIALOG ""
     @3,19 HEIGHT 19 WIDTH 39

     @1,9
     ?? "The Event Trapper"
     PAINTCANVAS ATTRIBUTE 112 1,9,1,26

     @3,12
     ?? "Version 1.1"
     PAINTCANVAS ATTRIBUTE 112 3,12,3,22

     @5,9
     ?? "By Joshua Evans"
     PAINTCANVAS ATTRIBUTE 112 5,10,5,24

     @7,3
     ?? "Contributions by Dan Bernstein"
     PAINTCANVAS ATTRIBUTE 112 5,10,5,24     


     @9,5
     ?? "Paradox Technical Support"
     PAINTCANVAS ATTRIBUTE 112 7,5,7,29

     @11,7
     ?? "Borland International"
     PAINTCANVAS ATTRIBUTE 112 9,7,9,27

     PUSHBUTTON @14,10 WIDTH 16
        "OK"
        CANCEL
        DEFAULT
        VALUE "OK"
        TAG "OK"
        TO OKValue
  ENDDIALOG

ENDPROC

; Key Events Proc 

PROC Get_Key_Events()
;Key trapping module to demonstrate key events
PRIVATE 
    KeyTrapper,
    Show_Event_Handle
    

  ECHO OFF
  SHOWPULLDOWN
  FORMAT("W78,AC", "Key Events") : "" : ""
  ENDMENU
  PROMPT "Press [F2] to terminate Key Event Trapping"
  MINIEDIT {Open} SELECT "KeyTrap"
  PGDN
  KeyTrapper = GETWINDOW()
  LockWindow["Title"] = ""
  WINDOW SETATTRIBUTES KeyTrapper FROM LockWindow

  
  WINDOW CREATE @4,52 WIDTH 25 HEIGHT 17 ATTRIBUTES LockDynArrayWindow 
  TO Show_Event_Handle
  SETMARGIN 2  
  ECHO NORMAL
  CURSOR OFF
  ? "Type        ="
  ? "KeyCode     ="
  ? "ScanCode    ="
  ? ""
  ? "Caps Lock   ="
  ? "Num Lock    ="
  ? "Scroll Lock ="
  ? "Alt         ="
  ? "Ctrl        ="
  ? "Left Alt    ="
  ? "Left Ctrl   ="
  ? "Left Shift  ="
  ? "Right Shift ="

  WINDOW SELECT KeyTrapper
  SETCANVAS Show_Event_Handle
  ECHO NORMAL
  WHILE (True)
    GETEVENT KEY "ALL" MOUSE "DOWN" TO Event
    IF Event["TYPE"] = "MOUSE" THEN
      BEEP
      LOOP
    ENDIF  
    IF Event["KEYCODE"] = -60 THEN
      QUITLOOP
    ENDIF     
    CANVAS OFF
    CURSOR OFF    
    SETMARGIN 16
    @0,16 CLEAR EOS
    @0,16
    ? Event["TYPE"]
    ? Event["KEYCODE"]
    ? Event["SCANCODE"]
    ? ""
    ? Event["CAPS LOCK"]
    ? Event["NUM LOCK"]    
    ? Event["SCROLL LOCK"]
    ? Event["ALT"]    
    ? Event["CTRL"]    
    ? Event["LEFT ALT"]    
    ? Event["LEFT CTRL"]
    ? Event["LEFT SHIFT"]    
    ? Event["RIGHT SHIFT"]    
    
    CANVAS ON
    Key = Event["KEYCODE"]
    IF Key <> ASC("ALTSPACE") AND 
       Key <> ASC("DOS") AND 
       Key <> ASC("DOSBIG") AND
       Key <> ASC("ORDERTABLE") AND
       Key <> ASC("MINIEDIT") AND
       Key <> ASC("ZOOM") AND
       Key <> ASC("ZOOMNEXT") AND
       Key <> ASC("WINNEXT") THEN
      EXECEVENT Event
    ENDIF  
  ENDWHILE    
  WINDOW SELECT KeyTrapper
  WINDOW CLOSE
  WINDOW SELECT Show_Event_Handle
  WINDOW CLOSE
ENDPROC  

; Mouse Events Proc 

PROC Get_Mouse_Events()
  ECHO OFF
  SHOWPULLDOWN 
  FORMAT("W78,AC", "Mouse Events") : "" : ""
  ENDMENU

  PROMPT "Press [F2] to terminate Mouse Event Trapping"
  LockWindow["Title"] = ""  
  WINDOW CREATE @2,2 ATTRIBUTES LockWindow TO MouseTextWin
  SETMARGIN 1
TEXT

This is the Mouse Event Trapping Module.
Move the mouse around, click, 
doubleclick, hold the mouse down for a
period of time to view the dynamic 
arrays created by these different 
actions. You may also press the various
keys described in the bottom of the 
window to the right to see how they 
change.
ENDTEXT  


  WINDOW CREATE @2,51 WIDTH 25 HEIGHT 20 ATTRIBUTES LockDynArrayWindow
  TO Show_Event_Handle
  SETMARGIN 2  
  CURSOR OFF
  ? "Type        ="
  ? "Action      ="
  ? "Row         ="
  ? "Col         ="
  ? "DoubleClick ="
  ? "Buttons     ="
  ? ""
  ? "Caps Lock   ="
  ? "Num Lock    ="
  ? "Scroll Lock ="
  ? "Alt         ="
  ? "Ctrl        ="
  ? "Left Alt    ="
  ? "Left Ctrl   ="
  ? "Left Shift  ="
  ? "Right Shift ="
      WINDOW SELECT MouseTextWin
      SETCANVAS Show_Event_Handle

  ECHO NORMAL
  WHILE (True)
    GETEVENT KEY "ALL" MOUSE "ALL" TO Event
    IF Event["TYPE"] = "KEY" THEN
      IF Event["KEYCODE"] = -60 THEN
        QUITLOOP
      ELSE
        BEEP
        LOOP
      ENDIF      
    ENDIF     
    CANVAS OFF
    CURSOR OFF    
    SETMARGIN 16
    @0,16 CLEAR EOS
    @0,16
    ? Event["TYPE"]
    ? Event["ACTION"]
    ? Event["ROW"]
    ? Event["COL"]
    ? Event["DOUBLECLICK"]
    ? Event["BUTTONS"]        
    ? ""
    ? Event["CAPS LOCK"]
    ? Event["NUM LOCK"]    
    ? Event["SCROLL LOCK"]
    ? Event["ALT"]    
    ? Event["CTRL"]    
    ? Event["LEFT ALT"]    
    ? Event["LEFT CTRL"]
    ? Event["LEFT SHIFT"]    
    ? Event["RIGHT SHIFT"]    

    CANVAS ON
  ENDWHILE    
  WINDOW SELECT MouseTextWin
  WINDOW CLOSE
  WINDOW SELECT Show_Event_Handle
  WINDOW CLOSE
ENDPROC  

; Message Events Proc 

PROC MessageMenu()
SHOWPULLDOWN
  "Choice_1" : "First Menu Selection - TAG = MENU1" : "MENU1"
    SUBMENU
      "Choice_1A" : "First SubMenu Selection from MenuChoice1"  : "MENU1A",
      "Choice_1B" : "Second SubMenu Selection from MenuChoice1" : "MENU1B"
    ENDSUBMENU,
  "Choice_2" : "Second Menu Selection" : "MENU2"
    SUBMENU
      "Choice_2A" : "First SubMenu Selection from MenuChoice2" : "MENU2A",
      "Choice_2B" : "Second SubMenu Selection from MenuChoice2" : "MENU2B"
    ENDSUBMENU,  
  "Choice_3" : "Third Menu Selection" : "MENU3"
    SUBMENU
      "Choice_3A" : "First SubMenu Selection from MenuChoice3" : "MENU3A",
      "Choice_3B" : "Second SubMenu Selection from MenuChoice3" : "MENU3B"
    ENDSUBMENU
ENDMENU
UNTIL "F1", "F2" 
ENDPROC

PROC CreateMessageWin()
  DYNARRAY MessageWinDynarray[]
  MessageWinDynarray["CANRESIZE"] = False
  MessageWinDynarray["CANMOVE"] = False
  MessageWinDynarray["CANVASHEIGHT"] = 18
  MessageWinDynarray["CANVASWIDTH"] = 39
  MessageWinDynarray["TITLE"] = "Message Events"
  
  
  WINDOW CREATE ATTRIBUTES MessageWinDynarray @2,2 WIDTH 41 HEIGHT 20 
  TO MessageWin
  SETMARGIN 1
TEXT
This is the Message Event Trapping 
Module.The following Events can be
demonstrated with the following
actions:

MenuSelect: Choose a Menu Choice

MenuKey   : Press [F1] while in the
            menu

Next      : Press [Ctrl-F4]

Maximize  : Click on Frame arrow 
            or Press [Shift-F5] twice

Close     : Click on Close Box or
            Press [Ctrl-F8]
ENDTEXT
ENDPROC  

PROC Get_Message_Events()
  ECHO OFF
  PROMPT "Press [F2] to terminate Message Event Trapping"
  MessageMenu()
  CreateMessageWin()
  LockDynarrayWindow["TITLE"] = "Floating Window"
  LockDynarrayWindow["CANVASHEIGHT"] = 18
  LockDynarrayWindow["CANVASWIDTH"] = 27
  WINDOW CREATE FLOATING @2,48 WIDTH 29 HEIGHT 20 ATTRIBUTES LockDynArrayWindow
  TO Show_Event_Handle
  SETCANVAS Show_Event_Handle
  WINDOW SELECT MessageWin
  SETMARGIN 2  
  CURSOR OFF
  ? "Type        ="
  ? "Message     ="
  ? "MenuTag     ="
  ? "KeyCode     ="
  ? "ScanCode    ="
  ? ""
  ? "Caps Lock   ="
  ? "Num Lock    ="
  ? "Scroll Lock ="
  ? "Alt         ="
  ? "Ctrl        ="
  ? "Left Alt    ="
  ? "Left Ctrl   ="
  ? "Left Shift  ="
  ? "Right Shift ="

  ECHO NORMAL
  WHILE (True)
    GETEVENT KEY -60 MESSAGE "ALL" TO Event
    IF Event["TYPE"] = "KEY" THEN
      CLEARPULLDOWN
      QUITLOOP
    ENDIF     
    CANVAS OFF
    CURSOR OFF    
    SETMARGIN 16
    @0,16 CLEAR EOS
    @0,16
    ? Event["TYPE"]
    ? Event["MESSAGE"]

    IF Event["MESSAGE"] = "MENUSELECT" OR Event["MESSAGE"] = "MENUKEY" THEN
      ? Event["MENUTAG"]
    ENDIF
    
    IF Event["MESSAGE"] = "MENUKEY" THEN
      ? Event["KEYCODE"]
      ? Event["SCANCODE"]
      ? ""
      ? Event["CAPS LOCK"]
      ? Event["NUM LOCK"]    
      ? Event["SCROLL LOCK"]
      ? Event["ALT"]    
      ? Event["CTRL"]    
      ? Event["LEFT ALT"]    
      ? Event["LEFT CTRL"]
      ? Event["LEFT SHIFT"]    
      ? Event["RIGHT SHIFT"]    
    ENDIF
    CANVAS ON

    IF Event["MESSAGE"] = "NEXT" THEN
      IF GETWINDOW() <> MessageWin THEN
        WINDOW SELECT MessageWin 
      ELSE
        WINDOW SELECT Show_Event_Handle
      ENDIF
      LOOP
    ENDIF      

    EXECEVENT Event
  ENDWHILE    
  IF ISWINDOW(MessageWin) THEN
    WINDOW SELECT MessageWin
    WINDOW CLOSE  
  ENDIF    
  WINDOW SELECT Show_Event_Handle
  WINDOW CLOSE  

ENDPROC  


; Dialog Triggers 


PROC TrainingDialogProc(Trigger, Tag, Event, Element)

    IF TRIGGER = "OPEN" THEN
      DialogHandle = GETWINDOW()
    ENDIF      

  IF Trigger = "EVENT" THEN
    CANCELDIALOG
    F2Pressed = True
    MESSAGE "Clearing Dialog session..."
    SLEEP 1000
    RETURN
  ENDIF    

  SETCANVAS Show_Event_Handle
  CANVAS OFF
  CURSOR OFF    
  SETMARGIN 17
  @0,16 CLEAR EOS
  @0,16
  ? Trigger
  ? Tag
  ? Event
  ? Element
  CANVAS ON
  WINDOW SELECT DialogHandle
  SETCANVAS DialogHandle

  SWITCH

    CASE Trigger = "UPDATE":
      MESSAGE "UPDATE has ocurred, wait for message to clear..."
      SLEEP 2000
      MESSAGE ""

    CASE Trigger = "CLOSE" AND F2Pressed = True : MESSAGE "" RETURN

    CASE Trigger <> "ARRIVE" AND Trigger <> "OPEN" AND Trigger <> "CLOSE":
      MESSAGE "Choose a Return Value"
      SHOWPOPUP "Return" @15, 52
        "True" : "Allow current Trigger to be processed and the Trigger cycle to continue" : "True",
        "False" : "Deny current Trigger and abort the current Trigger cycle" : "False"
      ENDMENU
      TO ReturnValue

      WINDOW SELECT DialogHandle
      IF ReturnValue = "Esc"  OR ReturnValue = "False" THEN
        RETURN FALSE
      ENDIF


    OTHERWISE: MESSAGE Trigger + " is an \"After\" Trigger (not deniable) - Press a key or click with mouse to continue"
      GETEVENT KEY "ALL" MOUSE "DOWN" TO ClearMessage
      MESSAGE ""

  ENDSWITCH
ENDPROC

  
PROC Get_Dialog_Triggers()

SHOWPULLDOWN
  FORMAT("W78,AC", "ShowDialog Triggers") : "" : ""
ENDMENU  

Dynarray OPENSCREEN[]
OPENSCREEN["MAXIMIZED"] = True
OPENSCREEN["MARGIN"] = 2
OPENSCREEN["CANCLOSE"] = False
OPENSCREEN["CANVASWIDTH"] = 78
OPENSCREEN["STYLE"] = 112
OPENSCREEN["CANRESIZE"] = False
OPENSCREEN["CANMAXIMIZE"] = False
OPENSCREEN["CANMOVE"] = False
OPENSCREEN["CANVASHEIGHT"] = 21
OPENSCREEN["HASSHADOW"] = False

WINDOW CREATE ATTRIBUTES OpenScreen TO WaitOpenHandle

TEXT

This is the ShowDialog Trigger Trapping module. It's designed to demonstrate 
all of the Triggers available under the new Paradox 4.0 ShowDialog command.

To use this module interact with the Dialog box as you normally would. Each
mouse or key action has the potential to begin a cycle of one or more 
Triggers. Follow the instructions diplayed by the message which appears 
whenever it is nessary for you to take special action.

When a deniable ("Before") Trigger occurs you will be given the option to 
return one of the two possible ShowDialog Return values (True,False). This 
allows you to test the effect of these codes while amidst a Trigger cycle. 
Pressing Escape while in the menu returns False to the Dialog box. The only 
exception to this is Update Triggers which will display a message only. Wait 
for this message to clear of its own accord, then proceed.

When a non-deniable ("After") Trigger occurs you will be notified of its 
occurrence and must press a key or click the mouse to go on the next Trigger 
in the cycle or return to the Dialog box. When messages no longer appear the 
cycle is over and you are back in the Dialog box.
ENDTEXT

PROMPT "Press any key to continue..."
ECHO NORMAL
GETEVENT KEY "ALL" MOUSE "DOWN" TO Retval
WINDOW CLOSE

PROMPT "Press [F2] or close dialog normally to terminate Dialog Trigger Trapping"

F2Pressed = False

ARRAY DialogArray[5]
FOR i FROM 1 TO 5
  DialogArray[i] = "Item " + STRVAL(i)
ENDFOR

DYNARRAY AdjustCanvas[]
AdjustCanvas["CanvasHeight"] = 6
Adjustcanvas["CanvasWidth"] = 32
AdjustCanvas["CanClose"] = False
AdjustCanvas["CanResize"] = False
AdjustCanvas["CanMaximize"] = False
AdjustCanvas["CanMove"] = False

WINDOW CREATE @2,44 WIDTH 34 HEIGHT 8 ATTRIBUTES AdjustCanvas
TO Show_Event_Handle
SETMARGIN 2  
ECHO NORMAL
CURSOR OFF
? "TriggerType  ="
? "TagValue     ="
? "EventValue   ="
? "ElementValue ="

SHOWDIALOG ""
  PROC "TrainingDialogProc" KEY "F2" Trigger "ALL" 
  @2,1 HEIGHT 21 WIDTH 40
  FRAME FROM 12,1 TO 15, 36
  FRAME FROM 12,1 TO 18,36

  @13,3 ?? "The OK button has the OK setting"
  @14,3 ?? "and is the DEFAULT button."
  @16,3 ?? "The CANCEL button has the CANCEL"
  @17,3 ?? "setting."

  LABEL
    @0,1 "~P~ickArray"
    FOR "PickArrayTag"
    
  PICKARRAY
    @1,2 HEIGHT 3 WIDTH 12
    DialogArray
    TAG "PickArrayTag"
    TO PickArrayVar
    
  LABEL
    @5,1 "~C~heckBoxes"
    FOR "CheckBoxTag"

  CHECKBOXES
    @6,1 HEIGHT 3 WIDTH 13
    TAG "CheckBoxTag"
    "Box1" TO CheckBox1Var,
    "Box2" TO CheckBox2Var,
    "Box3" TO CheckBox3Var


  LABEL 
    @0,19 "~R~adio Buttons"
    FOR "RadioButtonTag"
    
  RADIOBUTTONS
    @1,19 HEIGHT 3 WIDTH 13
    "Option1", "Option2", "Option3"
    TAG "RadioButtonTag"
    TO RadioButtonVar

  LABEL
    @5,19 "~A~ccept Box"
    FOR "AcceptTag"

  ACCEPT
    @6,20 WIDTH 12
    "N"
    MIN 0 MAX 10
    TAG "AcceptTag"
    TO AcceptVar

  LABEL 
    @7,19 "~S~lider"
    FOR "SliderTag"

  SLIDER
    @8,20
    HORIZONTAL
    LENGTH 12
    MIN 0 MAX 10
    ARROWSTEP 1
    PAGESTEP 2
    TAG "SliderTag"
    TO AcceptVar

  PUSHBUTTON @10,7 WIDTH 10
    "OK"
    OK
    DEFAULT
    VALUE "ACCEPT"
    TAG "OKButton"
    TO OKButtonVar

  PUSHBUTTON @10,18 WIDTH 10
    "CANCEL"
    CANCEL
    VALUE "CANCEL"
    TAG "CANCELButton"
    TO CANCELButtonVar

ENDDIALOG    
WINDOW CLOSE
ENDPROC

; Wait Triggers 


PROC Get_Wait_Triggers()
ECHO OFF
DYNARRAY WaitAttributes[]
WaitAttributes["HasFrame"] = False
WaitAttributes["HasShadow"] = False
WaitAttributes["Height"] = 23
WaitAttributes["Width"] = 48
WaitAttributes["OriginCol"]= 0
WaitAttributes["OriginRow"] = 1
WaitAttributes["CanResize"] = False
WaitAttributes["CanMove"] = False
WaitAttributes["CanClose"] = False
WaitAttributes["CanMaximize"] = False

CoEdit "Date"
WaitImage = GETWINDOW()
WINDOW SETATTRIBUTES WaitImage FROM WaitAttributes
FORMKEY
WINDOW SETATTRIBUTES GETWINDOW() FROM WaitAttributes

DYNARRAY ShowTriggerWin[]
WINDOW CREATE @1,48 WIDTH 32 HEIGHT 23 ATTRIBUTES LockDynArrayWindow
TO Show_Event_Handle

SETCANVAS Show_Event_Handle
SETMARGIN 1
CURSOR OFF
IF Display = "FullDynArray" THEN
  ? "TriggerType  ="
  ? "CycleNumber  ="
  ? "EventType    ="
  ? "KeyCode      ="
  ? "ScanCode     ="
  ? "Action       ="
  ? "Row          ="
  ? "Col          ="
  ? "DoubleClick  ="
  ? "Buttons      ="        
  ? "Caps Lock    ="
  ? "Num Lock     ="
  ? "Scroll Lock  ="
  ? "Alt          ="
  ? "Ctrl         ="
  ? "Left Alt     ="
  ? "Left Ctrl    ="
  ? "Left Shift   ="
  ? "Right Shift  ="
ENDIF
SHOWPULLDOWN 
  FORMAT("W78,AC","Wait Triggers") : "" : ""
ENDMENU


Dynarray OPENSCREEN[]
OPENSCREEN["MAXIMIZED"] = True
OPENSCREEN["MARGIN"] = 2
OPENSCREEN["CANCLOSE"] = False
OPENSCREEN["CANVASWIDTH"] = 78
OPENSCREEN["STYLE"] = 112
OPENSCREEN["CANRESIZE"] = False
OPENSCREEN["CANMAXIMIZE"] = False
OPENSCREEN["CANMOVE"] = False
OPENSCREEN["CANVASHEIGHT"] = 21
OPENSCREEN["HASSHADOW"] = False

WINDOW CREATE ATTRIBUTES OpenScreen TO WaitOpenHandle


IF Display = "FullDynArray" THEN
TEXT

This is the Incremental Display version of the Wait Trigger Trapping module.
It's designed to demonstrate most of the Triggers available under the new 
Event-Driven version of the 4.0 Wait command.

To use this module simply move around the workspace as you normally would
through a multi-table form. Each mouse or key action has the potential to 
begin a cycle of one or more Triggers. Follow the instructions diplayed by 
the message which appears whenever it is nessary for you to take special 
action.

When a deniable ("Before") Trigger occurs you will be given the option to 
return one of the three possible Wait Return Codes (0,1,2). This gives you 
the ability to test the effect of these codes while amidst a Trigger cycle. 
Pressing Escape while in the menu returns a 1 to the Wait. 

When a non-deniable ("After") Trigger occurs you will be notified of its 
occurrence and must press a key or click the mouse to go on the next Trigger 
in the cycle or return to the Wait. When messages no longer appear the cycle 
is over and you are back in the Wait.
ENDTEXT
ELSE
TEXT

This is the Short List version of the Wait Trigger Trapping module. It's 
designed to demonstrate most of the Triggers available under the new 
Event-Driven version of the 4.0 Wait command.

To use this module simply move around the workspace as you normally would
through a multi-table form. Each mouse or key action has the potential to 
begin a cycle of one or more Triggers. Each Trigger Cycle initiated by a 
single user action will be displayed as a list. As new cycles occur the old
one will be cleared, so that the list always reflects the last complete 
trigger cycle.

ENDTEXT

ENDIF

PROMPT "Press any key to continue..."
ECHO NORMAL
GETEVENT KEY "ALL" MOUSE "DOWN" TO Retval
WINDOW CLOSE

FullDynArrayPrompt = "Press [F2] or Return [2] from any Trigger to terminate Wait"
ShortListPrompt = "Press [F2] to terminate Wait"

PROMPT IIF(Display = "FullDynArray", FullDynArrayPrompt, ShortListPrompt)
SETCANVAS Show_Event_Handle
cyclenum=-1
WAIT WORKSPACE
PROC IIF(Display = "FullDynArray","FullDynWaitProc","ShortListWaitProc")
 KEY -60 Trigger "ALL"
ENDWAIT
DO_IT!
IF SYSMODE() <> "Main" THEN
  UNDO
  DO_IT!
ENDIF  
ECHO OFF
CLEARIMAGE
WINDOW CLOSE
ENDPROC

PROC FullDynWaitProc(Trigger, Event, Cycle)
IF Event["Type"] = "KEY" THEN
  IF Event["KEYCODE"] = -60 THEN
    RETURN 2
  ENDIF  
ENDIF  

CANVAS OFF
CURSOR OFF    
SETMARGIN 16
@0,16 CLEAR EOS
? Trigger
? Cycle
? Event["TYPE"]
IF Event["TYPE"] = "KEY" THEN
    ? Event["KEYCODE"]
    ? Event["SCANCODE"]
ELSE
    ? ""
    ? ""
ENDIF    
IF Event["Type"] = "MOUSE" THEN
    ? Event["ACTION"]
    ? Event["ROW"]
    ? Event["COL"]
    ? Event["DOUBLECLICK"]
    ? Event["BUTTONS"]        
ELSE
    FOR i FROM 1 TO 5
      ? ""
    ENDFOR
ENDIF          
IF Event["TYPE"] = "KEY" OR Event["TYPE"] = "MOUSE" THEN
    ? Event["CAPS LOCK"]
    ? Event["NUM LOCK"]    
    ? Event["SCROLL LOCK"]
    ? Event["ALT"]    
    ? Event["CTRL"]    
    ? Event["LEFT ALT"]    
    ? Event["LEFT CTRL"]
    ? Event["LEFT SHIFT"]    
    ? Event["RIGHT SHIFT"]    
ENDIF
CANVAS ON
  IF SUBSTR(Trigger,1,6) <> "ARRIVE" THEN
      MESSAGE "Choose a Return Code"
      SHOWPOPUP "Return" @10,31
        "0" : "Allow current Trigger and subsequent Triggers to be processed" : "0",
        "1" : "Deny current Trigger and subsequent Triggers" : "1",
        "2" : "Deny current Trigger and terminate Wait" : "2"
      ENDMENU
      TO ReturnValue
    IF Retval = False THEN
      RETURN 1
    ELSE 
      RETURN NUMVAL(ReturnValue)
    ENDIF
  ELSE
    MESSAGE Trigger + " is an \"After\" Trigger (not deniable) - Press a key or click with mouse to continue"
    GETEVENT KEY "ALL" MOUSE "DOWN" TO ClearMessage
    MESSAGE ""
  ENDIF
RETURN 0
ENDPROC

PROC ShortListWaitProc(Trigger, Event, Cycle)
IF Event["Type"] = "KEY" THEN
  IF Event["KEYCODE"] = -60 THEN
    RETURN 2
  ENDIF  
ENDIF  

  if Cycle<>cyclenum then
    cyclenum = Cycle
    clear
    @0,0
    ? "Cycle Number: ", Cycle 
    ? ""
  endif
  ? Trigger 
  return 0
endproc

PROC CLOSED Main()
MENU {} {Desktop} {Empty}

DYNARRAY GlobalColors[]
GlobalColors["8"] = 112
SETCOLORS FROM GlobalColors

Dynarray SPLASHWIN[]
SPLASHWIN["MAXIMIZED"] = True
SPLASHWIN["CANCLOSE"] = False
SPLASHWIN["ORIGINROW"] = 1
SPLASHWIN["CANRESIZE"] = False
SPLASHWIN["CANMAXIMIZE"] = False
SPLASHWIN["CANMOVE"] = False
SPLASHWIN["HASFRAME"] = False
SPLASHWIN["ORIGINCOL"] = 0
SPLASHWIN["WIDTH"] = 73
SPLASHWIN["HASSHADOW"] = False
SPLASHWIN["HEIGHT"] = 18

WINDOW CREATE ATTRIBUTES SPLASHWIN TO SPLASHHANDLE
CURSOR OFF
CANVAS OFF
CLEAR

STYLE ATTRIBUTE 3
@0,0
SETMARGIN 0
TEXT

۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱
۱

ENDTEXT
STYLE ATTRIBUTE 8
SETMARGIN 27
@8,0
TEXT
                         
                         

ENDTEXT
STYLE ATTRIBUTE 8
SETMARGIN 22
@13,0
TEXT
                                    
                                    

ENDTEXT

STYLE ATTRIBUTE 63                 
@7,25 ??" ͻ "
@8,25 ??"                         "
@9,25 ??" ͼ "

STYLE ATTRIBUTE 63
@12,20 ??" ͻ "
@13,20 ??"                                    "
@14,20 ??" ͼ "

STYLE ATTRIBUTE 15
@13,23 ?? " A PARADOX 4.0 INTERACTIVE DEMO "
@8,28 ?? "  THE EVENT TRAPPER  "
CANVAS ON


DYNARRAY LockDynArrayWindow[]
LockDynArrayWindow["CanClose"] = False
LockDynArrayWindow["CanResize"] = False
LockDynArrayWindow["CanMaximize"] = False
LockDynArrayWindow["CanMove"] = False

DYNARRAY LockWindow[]
LockWindow["OriginRow"] = 2
LockWindow["OriginCol"] = 2
LockWindow["CanClose"] = False
LockWindow["CanResize"] = False
LockWindow["CanMaximize"] = False
LockWindow["CanMove"] = False
LockWindow["Height"] = 20
LockWindow["Width"] = 44
LockWindow["CanvasWidth"] = 42
LockWindow["CanvasHeight"] = 18

DYNARRAY Menu_Action_Array[]
Menu_Action_Array["About"] = "About"
Menu_Action_Array["Message"] = "Get_Message_Events"
Menu_Action_Array["Mouse"]   = "Get_Mouse_Events"
Menu_Action_Array["Key"]     = "Get_Key_Events"
Menu_Action_Array["Dialog"]  = "Get_Dialog_Triggers"
Menu_Action_Array["Wait"] = "Get_Wait_Triggers"

WHILE (True)
ECHO OFF
WINDOW MOVE SplashHandle TO 1,0
ECHO NORMAL
SHOWPULLDOWN
ENDMENU
SHOWPULLDOWN
  "" : "" : ""
    SUBMENU
      "About Trapper" : "Information about The Event Trapper" : "About"
    ENDSUBMENU,  
      "Events" : "Trap for Key, Mouse, or Message Events" :
        SUBMENU
          "Key"     : "Trap for Key Events only"  : "Key",
          "Mouse"   : "Trap for Mouse Events only": "Mouse",
          "Me~s~sage" : "Trap for Message Events only" : "Message"
        ENDSUBMENU,
      "Triggers" : "Trap for ShowDialog or Wait Triggers" :  
        SUBMENU
          "Wait": "Trap for Events within a Wait, emphesis on Triggers"   : 
            SUBMENU
              "Complete Incremental Display" : "Incrementally view each Trigger and its corresponding Dynarray" : "Wait1",
              "Short List of Trigger Cycles" : "Short list of complete Trigger Cycle for each user action" : "Wait2"
            ENDSUBMENU,
          "ShowDialog": "Trap for Events within a Dialog, emphesis on Triggers" : "Dialog"
        ENDSUBMENU
ENDMENU  
GETMENUSELECTION TO MenuSelection

IF NOT Retval THEN
  QUITLOOP
ENDIF  

SWITCH
  CASE MenuSelection = "Wait1": 
    Display = "FullDynArray" 
    MenuSelection = "Wait"
  CASE MenuSelection = "Wait2":
    Display = "ShortList"
    MenuSelection = "Wait"
ENDSWITCH

WINDOW MOVE SplashHandle TO -100,-100

EXECPROC Menu_Action_Array[MenuSelection]
ENDWHILE
ECHO OFF
ALTSPACE {Desktop} {Empty}
SETCOLORS DEFAULT
ENDPROC


Main()
